@@ -8,7 +8,7 @@ from .models import Article
 class SampleTestCase(TestCase):
     fixtures = ['fixture1.json', 'fixture2.json']
 
-    def testClassFixtures(self):
+    def test_class_fixtures(self):
         "Test cases can load fixture objects into models defined in packages"
         self.assertEqual(Article.objects.count(), 3)
         self.assertQuerysetEqual(
