@@ -1,3 +1,4 @@
+import os
 import re
 from io import StringIO
 from unittest import mock, skipUnless
@@ -52,7 +53,7 @@ class InspectDBTestCase(TestCase):
         output = out.getvalue()
 
         def assertFieldType(name, definition):
-            out_def = re.search(r'^\s*%s = (models.*)$' % name, output, re.MULTILINE).groups()[0]
+            out_def = re.search(r'^\s*%s = (models.*)$' % name, output, re.MULTILINE)[1]
             self.assertEqual(definition, out_def)
 
         return assertFieldType
@@ -85,6 +86,15 @@ class InspectDBTestCase(TestCase):
         elif not connection.features.interprets_empty_strings_as_nulls:
             assertFieldType('uuid_field', "models.CharField(max_length=32)")
 
+    @skipUnlessDBFeature('can_introspect_json_field', 'supports_json_field')
+    def test_json_field(self):
+        out = StringIO()
+        call_command('inspectdb', 'inspectdb_jsonfieldcolumntype', stdout=out)
+        output = out.getvalue()
+        if not connection.features.interprets_empty_strings_as_nulls:
+            self.assertIn('json_field = models.JSONField()', output)
+        self.assertIn('null_json_field = models.JSONField(blank=True, null=True)', output)
+
     def test_number_field_types(self):
         """Test introspection of various Django field types"""
         assertFieldType = self.make_field_type_asserter()
@@ -118,11 +128,19 @@ class InspectDBTestCase(TestCase):
             assertFieldType('pos_int_field', "models.IntegerField()")
 
         if connection.features.can_introspect_positive_integer_field:
+            if connection.features.can_introspect_big_integer_field:
+                assertFieldType('pos_big_int_field', 'models.PositiveBigIntegerField()')
+            else:
+                assertFieldType('pos_big_int_field', 'models.PositiveIntegerField()')
             if connection.features.can_introspect_small_integer_field:
                 assertFieldType('pos_small_int_field', "models.PositiveSmallIntegerField()")
             else:
                 assertFieldType('pos_small_int_field', "models.PositiveIntegerField()")
         else:
+            if connection.features.can_introspect_big_integer_field:
+                assertFieldType('pos_big_int_field', 'models.BigIntegerField()')
+            else:
+                assertFieldType('pos_big_int_field', 'models.IntegerField()')
             if connection.features.can_introspect_small_integer_field:
                 assertFieldType('pos_small_int_field', "models.SmallIntegerField()")
             else:
@@ -380,9 +398,9 @@ class InspectDBTransactionalTests(TransactionTestCase):
                     sepal_length real,
                     sepal_width real
                 ) SERVER inspectdb_server OPTIONS (
-                    filename '/dev/null'
+                    filename %s
                 )
-            ''')
+            ''', [os.devnull])
         out = StringIO()
         foreign_table_model = 'class InspectdbIrisForeignTable(models.Model):'
         foreign_table_managed = 'managed = False'
