@@ -1,7 +1,7 @@
 from unittest import mock
 
 from django.db import transaction
-from django.test import TestCase, skipUnlessDBFeature
+from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature
 
 from .models import (
     Article, InheritedArticleA, InheritedArticleB, Publication, User,
@@ -104,6 +104,12 @@ class ManyToManyTests(TestCase):
         a5.authors.remove(user_2.username)
         self.assertQuerysetEqual(a5.authors.all(), [])
 
+    def test_add_remove_invalid_type(self):
+        msg = "Field 'id' expected a number but got 'invalid'."
+        for method in ['add', 'remove']:
+            with self.subTest(method), self.assertRaisesMessage(ValueError, msg):
+                getattr(self.a1.publications, method)('invalid')
+
     def test_reverse_add(self):
         # Adding via the 'other' end of an m2m
         a5 = Article(headline='NASA finds intelligent life on Mars')
@@ -152,6 +158,14 @@ class ManyToManyTests(TestCase):
         with self.assertNumQueries(1):
             self.a1.publications.add(self.p1, self.p2)
 
+    @skipIfDBFeature('supports_ignore_conflicts')
+    def test_add_existing_different_type(self):
+        # A single SELECT query is necessary to compare existing values to the
+        # provided one; no INSERT should be attempted.
+        with self.assertNumQueries(1):
+            self.a1.publications.add(str(self.p1.pk))
+        self.assertEqual(self.a1.publications.get(), self.p1)
+
     @skipUnlessDBFeature('supports_ignore_conflicts')
     def test_slow_add_ignore_conflicts(self):
         manager_cls = self.a1.publications.__class__
@@ -455,6 +469,19 @@ class ManyToManyTests(TestCase):
         self.a4.publications.set([], clear=True)
         self.assertQuerysetEqual(self.a4.publications.all(), [])
 
+    def test_set_existing_different_type(self):
+        # Existing many-to-many relations remain the same for values provided
+        # with a different type.
+        ids = set(Publication.article_set.through.objects.filter(
+            article__in=[self.a4, self.a3],
+            publication=self.p2,
+        ).values_list('id', flat=True))
+        self.p2.article_set.set([str(self.a4.pk), str(self.a3.pk)])
+        new_ids = set(Publication.article_set.through.objects.filter(
+            publication=self.p2,
+        ).values_list('id', flat=True))
+        self.assertEqual(ids, new_ids)
+
     def test_assign_forward(self):
         msg = (
             "Direct assignment to the reverse side of a many-to-many set is "
